/********************************************************************
 * COPYRIGHT -- Microsoft
 ********************************************************************
 * Program: MultiAxis
 * File: MultiAxis.c
 * Author: mus
 * Created: March 05, 2015
 ********************************************************************
 * Implementation of program MultiAxis
 ********************************************************************/


#include "string.h"

#ifdef _DEFAULT_INCLUDES
	#include <AsDefault.h>
#endif


/*ʹע⣺*/
/*ÿĿͬ޸
ϵǷswitch on
ֹͣǷswitch off*/
void _INIT MultiAxisINIT(void)
{
	/* Init bit reset */
	initOK = 0;
		
	for(i = 0; i < MULTI_AXIS_NUM; i ++)
	{
		/* Get AxisObject according to access-name in NC-Deployment-Table */
		ncStatus = ncaccess(ncACP10MAN, MULTI_AXIS_NAME[i], &axisObj[i]);
			
		if ( (ncStatus != ncOK) && (ncStatus != 10600) )
		{
			/* The NC object pointer is invalid */
			gMultiAxis[i].alarm.errorNumber = ncStatus;
			strcpy(gMultiAxis[i].alarm.errorText[0], "Error to access name in NC-Deployment-Table");
			return;
		}
		/* Get the pointer of NC manager structure */
		pAxisData = (ACP10AXIS_typ*)axisObj[i];
		if(pAxisData == 0)
		{
			strcpy(gMultiAxis[i].alarm.errorText[0], "Error to get pAxisData");
			return;
		} 
	
		if ( pAxisData->size != sizeof(ACP10AXIS_typ) )
		{
			/* The NC data type is not compatible with NC manager */
			gMultiAxis[i].alarm.errorNumber = pAxisData->size;
			strcpy(gMultiAxis[i].alarm.errorText[0], "NC data type is not compatible with NC manager");
			return;
		}				
		if ( ( pAxisData->sw_version.nc_manager & 0xFFF0 ) != ( ACP10MAN_H_VERSION & 0xFFF0 ) )
		{
			/* The version is not compatible with NC manager */
			gMultiAxis[i].alarm.errorNumber = pAxisData->sw_version.nc_manager;
			strcpy(gMultiAxis[i].alarm.errorText[0], "The version is not compatible with NC manager");
			return;
		}
	
		pAxisData->message.text.parameter.columns   = sizeof(gMultiAxis[i].alarm.errorText[0]);
		pAxisData->message.text.parameter.data_len  = sizeof(gMultiAxis[i].alarm.errorText);
		pAxisData->message.text.parameter.data_adr  = (UDINT)&gMultiAxis[i].alarm.errorText;
		strcpy((char*)&pAxisData->message.text.parameter.data_modul[0], MULTI_AXIS_ERROR_TEXT_NAME);
	
		gMultiAxis[i].status.errorStep = 0;		/* Reset error step */
		
		step[i] = STEP_WAIT_NET_INIT;
		
		/*˶ʼ*/
		gMultiAxis[i].para.velocity		= 3000;
		gMultiAxis[i].para.jogVelocity	= 3000;
		gMultiAxis[i].para.acceleration	= 10000;
		gMultiAxis[i].para.deceleration	= 10000;
		gMultiAxis[i].para.absPosition	= 0;
		gMultiAxis[i].para.relPosition	= 0;
		
	}
	
	initOK = 1;

}

void _CYCLIC MultiAxisCYCLIC(void)
{
	if (initOK == 0)    return;     /*ʼ*/
	
	for(i = 0; i < MULTI_AXIS_NUM; i ++)
	{
	
		/* Get the pointer of NC manager structure */
		pAxisData = (ACP10AXIS_typ*)axisObj[i];

		/* Check for an axis error */		
		if (pAxisData->message.record.number != 0 && gMultiAxis[i].status.errorStep == 0)
		{		
			gMultiAxis[i].status.errorStep = step[i];
			step[i] = STEP_ERROR_STOP;
		}/* if (ncTRUE == pAxisData->monitor.status.error) */
		/* Refresh status data */
		gMultiAxis[i].alarm.errorNumber   = pAxisData->message.record.number;
		gMultiAxis[i].status.actPosition	= pAxisData->monitor.s;
		gMultiAxis[i].status.actVelocity	= pAxisData->monitor.v;
		gMultiAxis[i].status.moveMode	    = pAxisData->move.mode;
		gMultiAxis[i].status.ctrlReady	= pAxisData->controller.status;
		gMultiAxis[i].status.homeReady	= pAxisData->move.homing.status.ok;
		gMultiAxis[i].status.inPosition	= pAxisData->move.basis.status.in_pos;
		gMultiAxis[i].status.step		    = step[i];
    
		/* State sequence for handling the NC Object */
		switch (step[i])
		{
			/* Wait until the network is initialized */
			case STEP_WAIT_NET_INIT:
				if( pAxisData->network.init == ncTRUE )
				{
					step[i] = STEP_INIT_STOP;
				}
				else if (pAxisData->network.init_allowed == ncTRUE) /*acoposϵϵ-൱ڴacp10cfgexcute net init automatic after acopos reset */
				{
					/*Network initialization allowed*/ 
					step[i] = STEP_NETWORK_INIT_ALLOWED;
				}
				break;
			/* Network initialization */
			case STEP_NETWORK_INIT_ALLOWED:
				ncStatus = ncaction(axisObj[i], ncNETWORK, ncINIT);
				if (ncStatus == ncOK)
				{
					step[i] = STEP_WAIT_NETWORK_INIT_ALLOWED;
				}
				break;
			/* Wait network initialization completed */
			case STEP_WAIT_NETWORK_INIT_ALLOWED:
				if (pAxisData->network.init == ncTRUE)
				{
					step[i] = STEP_INIT_STOP;
				}
				break;
			case STEP_INIT_STOP:
				if(pAxisData->move.mode == ncOFF) 
				{
					step[i] = STEP_INIT_SWITCH_OFF;
					break;
				}
				pAxisData->move.stop.index.command = 0;
				ncStatus = ncaction(axisObj[i],ncMOVE,ncSTOP);
				if(ncStatus == ncOK) 
				{
					step[i] = SETP_WAIT_INIT_STOP;
				}
				break;
			case SETP_WAIT_INIT_STOP:
				if(pAxisData->move.mode == ncOFF)
				{
					step[i] = STEP_INIT_SWITCH_OFF;
				}
				break;
			/* Switch controller off for initialization */
			case STEP_INIT_SWITCH_OFF:
				if (pAxisData->controller.status == ncOFF)
				{
					step[i] = STEP_GLOBAL_INIT;
					break;
				}
				ncStatus = ncaction(axisObj[i], ncCONTROLLER, ncSWITCH_OFF);
				if (ncStatus == ncOK)
				{
					step[i] = STEP_WAIT_INIT_SWITCH_OFF;
				}	
				break;
			/* Wait for controller off for initialization */
			case STEP_WAIT_INIT_SWITCH_OFF:
				if (pAxisData->controller.status == ncOFF)
				{
					step[i] = STEP_GLOBAL_INIT;
				}
				break;
			/* Global initialization */
			case STEP_GLOBAL_INIT:
				/*ִйʼִУֹhomingʧ*/
				if(pAxisData->global.init == ncTRUE)    
				{
					step[i] = STEP_INIT_HOMING;
					break;
				}
            
				ncStatus = ncaction(axisObj[i], ncGLOBAL, ncINIT); 
				if (ncStatus == ncOK)
				{
					step[i] = STEP_WAIT_GLOBAL_INIT;
				}               
				break;
			/* Wait for end of global initialization */
			case STEP_WAIT_GLOBAL_INIT:
				if (pAxisData->global.init == ncTRUE)
				{
					step[i] = STEP_INIT_HOMING;
				}
				break;
			/* Start homing procedure ѡǷִһ*/
			case STEP_INIT_HOMING:
				/*homing okִ*/
				if(pAxisData->move.homing.status.ok == ncTRUE)
				{
					step[i] = STEP_COMMAND;
					break;
				}

				pAxisData->move.homing.parameter.mode = ncDIRECT;
				ncStatus = ncaction(axisObj[i], ncHOMING, ncINIT + ncSTART);
				if (ncStatus == ncOK)
				{
					step[i] = STEP_WAIT_INIT_HOMING;
				}               
				break;
		
			/* Wait for end of homing procedure */
			case STEP_WAIT_INIT_HOMING:
				if(pAxisData->move.homing.status.ok == ncTRUE)
				{
					step[i] = STEP_INIT_SWITCH_ON;    /*ѡ STEP_COMMAND  STEP_INIT_SWITCH_ON*/
				}
				break;	
			/* Switch controller on */
			case STEP_INIT_SWITCH_ON:
				ncStatus = ncaction(axisObj[i], ncCONTROLLER, ncSWITCH_ON);
				if (ncStatus == ncOK)
				{
					step[i] = STEP_WAIT_INIT_SWITCH_ON;
				}
				break;
			/* Wait for controller on */
			case STEP_WAIT_INIT_SWITCH_ON:
				if (pAxisData->controller.status == ncON)
				{
					step[i] = STEP_COMMAND;
				}
				break;	       
			case STEP_COMMAND:
				if (gMultiAxis[i].cmd.ctrlOff == 1)
				{
					gMultiAxis[i].cmd.ctrlOff = 0;
					step[i] = STEP_SWITCH_OFF;
				}
				else if (gMultiAxis[i].cmd.ctrlOn == 1)
				{
					gMultiAxis[i].cmd.ctrlOn = 0;
					step[i] = STEP_SWITCH_ON;
				}
				else if (gMultiAxis[i].cmd.stop == 1)
				{
					gMultiAxis[i].cmd.stop = 0;
					step[i] = STEP_STOP_MOVE;
				}
				else if (gMultiAxis[i].cmd.posMove == 1)
				{
					gMultiAxis[i].cmd.posMove = 0;
					step[i] = STEP_POS_MOVE;
				}
				else if (gMultiAxis[i].cmd.negMove == 1)
				{
					gMultiAxis[i].cmd.negMove = 0;
					step[i] = STEP_NEG_MOVE;
				}
				else if (gMultiAxis[i].cmd.posJog == 1)
				{
					step[i] = STEP_POS_JOG;
				}
				else if (gMultiAxis[i].cmd.negJog == 1)
				{
					step[i] = STEP_NEG_JOG;
				}
				else if (gMultiAxis[i].cmd.absMove == 1) 
				{
					gMultiAxis[i].cmd.absMove = 0;
					step[i] = STEP_ABS_MOVE;
				}
				else if (gMultiAxis[i].cmd.relMove == 1)
				{
					gMultiAxis[i].cmd.relMove = 0;
					step[i] = STEP_REL_MOVE;
				}
				else if (gMultiAxis[i].cmd.home == 1)
				{
					gMultiAxis[i].cmd.home = 0;   
					step[i] = STEP_HOME;   
				}
				break;
			/* Switch controller on */
			case STEP_SWITCH_ON:
				if (pAxisData->controller.status == ncON)
				{
					step[i] = STEP_COMMAND;
					break;
				}
            
				ncStatus = ncaction(axisObj[i], ncCONTROLLER, ncSWITCH_ON);
				if (ncStatus == ncOK)
				{
					step[i] = STEP_WAIT_SWITCH_ON;
				}
				break;
			/* Wait for controller on */
			case STEP_WAIT_SWITCH_ON:
				if (pAxisData->controller.status == ncON)
				{
					step[i] = STEP_COMMAND;
				}
				break;	
			/* Start movement in positive direction */
			case STEP_POS_MOVE:
				pAxisData->move.basis.parameter.v_pos = gMultiAxis[i].para.velocity;
				pAxisData->move.basis.parameter.a1_pos = gMultiAxis[i].para.acceleration;
				pAxisData->move.basis.parameter.a2_pos = gMultiAxis[i].para.deceleration;
				ncStatus = ncaction(axisObj[i], ncPOS_MOVE, ncINIT + ncSTART);
				if (ncStatus == ncOK)
				{	
					step[i] = STEP_COMMAND;
				}	
				break;
			/* Start movement in negative direction */
			case STEP_NEG_MOVE:
				pAxisData->move.basis.parameter.v_neg = gMultiAxis[i].para.velocity;
				pAxisData->move.basis.parameter.a1_neg = gMultiAxis[i].para.acceleration;
				pAxisData->move.basis.parameter.a2_neg = gMultiAxis[i].para.deceleration;
				ncStatus = ncaction(axisObj[i], ncNEG_MOVE, ncINIT + ncSTART);
				if (ncStatus == ncOK)
				{	
					step[i] = STEP_COMMAND;
				}	
				break;
			/* Start jog in positive direction */
			case STEP_POS_JOG:
				pAxisData->move.basis.parameter.v_pos = gMultiAxis[i].para.jogVelocity;
				pAxisData->move.basis.parameter.a1_pos = gMultiAxis[i].para.acceleration;
				pAxisData->move.basis.parameter.a2_pos = gMultiAxis[i].para.deceleration;
				ncStatus = ncaction(axisObj[i], ncPOS_MOVE, ncINIT + ncSTART);
				if (ncStatus == ncOK)
				{	
					step[i] = STEP_WAIT_POS_JOG;
				}	
				break;
		
			/* Wait for stopping jog in positive direction */
			case STEP_WAIT_POS_JOG:
				/*other cmd reset jog cmd*/
				if(gMultiAxis[i].cmd.stop == 1 || gMultiAxis[i].cmd.ctrlOff == 1)
				{
					gMultiAxis[i].cmd.posJog = 0;
					gMultiAxis[i].cmd.negJog = 0;
					step[i] = STEP_STOP_MOVE;
					break;
				}
            
				if (gMultiAxis[i].cmd.posJog == 0)
				{
					step[i] = STEP_STOP_MOVE;
				}
				break;
		
			/* Start jog in negative direction */
			case STEP_NEG_JOG:
				pAxisData->move.basis.parameter.v_neg = gMultiAxis[i].para.jogVelocity;
				pAxisData->move.basis.parameter.a1_neg = gMultiAxis[i].para.acceleration;
				pAxisData->move.basis.parameter.a2_neg = gMultiAxis[i].para.deceleration;
				ncStatus = ncaction(axisObj[i], ncNEG_MOVE, ncINIT + ncSTART);
				if (ncStatus == ncOK)
				{	
					step[i] = STEP_WAIT_NEG_JOG;
				}	
				break;
		
			/* Wait for stopping jog in negative direction */
			case STEP_WAIT_NEG_JOG:
				/*other cmd reset jog cmd*/
				if(gMultiAxis[i].cmd.stop == 1 || gMultiAxis[i].cmd.ctrlOff == 1)
				{
					gMultiAxis[i].cmd.posJog = 0;
					gMultiAxis[i].cmd.negJog = 0;
					step[i] = STEP_STOP_MOVE;
					break;
				}
            
				if (gMultiAxis[i].cmd.negJog == 0)
				{
					step[i] = STEP_STOP_MOVE;
				}
				break;
			/* Start abolute movement */
			case STEP_ABS_MOVE:
				pAxisData->move.basis.parameter.s = gMultiAxis[i].para.absPosition;
				pAxisData->move.basis.parameter.v_pos = gMultiAxis[i].para.velocity;
				pAxisData->move.basis.parameter.v_neg = gMultiAxis[i].para.velocity;
				pAxisData->move.basis.parameter.a1_pos = gMultiAxis[i].para.acceleration;
				pAxisData->move.basis.parameter.a2_pos = gMultiAxis[i].para.deceleration;
				pAxisData->move.basis.parameter.a1_neg = gMultiAxis[i].para.acceleration;
				pAxisData->move.basis.parameter.a2_neg = gMultiAxis[i].para.deceleration;
			
				ncStatus = ncaction(axisObj[i], ncABS_MOVE, ncINIT + ncSTART);
				if (ncStatus == ncOK)
				{	
					step[i] = STEP_COMMAND;
				}	
				break;
			/* Start relative movement */
			case STEP_REL_MOVE:
				pAxisData->move.basis.parameter.s = gMultiAxis[i].para.relPosition;
				pAxisData->move.basis.parameter.v_pos = gMultiAxis[i].para.velocity;
				pAxisData->move.basis.parameter.v_neg = gMultiAxis[i].para.velocity;
				pAxisData->move.basis.parameter.a1_pos = gMultiAxis[i].para.acceleration;
				pAxisData->move.basis.parameter.a2_pos = gMultiAxis[i].para.deceleration;
				pAxisData->move.basis.parameter.a1_neg = gMultiAxis[i].para.acceleration;
				pAxisData->move.basis.parameter.a2_neg = gMultiAxis[i].para.deceleration;
			
				ncStatus = ncaction(axisObj[i], ncREL_MOVE, ncINIT + ncSTART);
				if (ncStatus == ncOK)
				{	
					step[i] = STEP_COMMAND;
				}	
				break;
			/* Start homing procedure */
			case STEP_HOME:
				pAxisData->move.homing.parameter.mode = gMultiAxis[i].para.homeMode;
				pAxisData->move.homing.parameter.s = gMultiAxis[i].para.homePosition;
            
				ncStatus = ncaction(axisObj[i], ncHOMING, ncINIT + ncSTART);
				if (ncStatus == ncOK)
				{
					step[i] = STEP_WAIT_HOME;
				}
				break;
		
			/* Wait for end of homing procedure */
			case STEP_WAIT_HOME:
				/*other cmd reset jog cmd*/
				if(gMultiAxis[i].cmd.stop == 1 || gMultiAxis[i].cmd.ctrlOff == 1)
				{
					gMultiAxis[i].cmd.posJog = 0;
					gMultiAxis[i].cmd.negJog = 0;
					step[i] = STEP_STOP_MOVE;
					break;
				}
            
				if (pAxisData->move.homing.status.ok == ncTRUE)
				{
					step[i] = STEP_COMMAND;
				}
				break;	
            
			/* Stop movement */
			case STEP_STOP_MOVE:
				pAxisData->move.stop.index.command = 0;
				ncStatus = ncaction(axisObj[i], ncMOVE, ncSTOP);
			
				if (ncStatus == ncOK)
				{					
					step[i] = STEP_WAIT_STOP_MOVE;
				}
				if (pAxisData->move.mode == ncOFF)
				{
					step[i] = STEP_COMMAND;
				}
				break;
		
			/* Wait for end of movement */
			case STEP_WAIT_STOP_MOVE:
				if (pAxisData->move.mode == ncOFF)
				{
					step[i] = STEP_COMMAND;
				}	
				break;
		
			/* Wait for end of movement before switch off */
			case STEP_SWITCH_OFF_CHECK:	
				if (pAxisData->move.mode != ncOFF) 
				{	
					pAxisData->move.stop.index.command = 0;				
					ncStatus = ncaction(axisObj[i], ncMOVE, ncSTOP);
					if (ncStatus == ncOK)
					{
						step[i] = STEP_SWITCH_OFF;
					}	
				}
				else
				{
					step[i] = STEP_SWITCH_OFF;   
				}	
				break;
		
			/* Switch controller off */
			case STEP_SWITCH_OFF:
				if (pAxisData->move.mode == ncOFF)
				{	
					ncStatus = ncaction(axisObj[i], ncCONTROLLER, ncSWITCH_OFF);
					if (ncStatus == ncOK)
					{
						step[i] = STEP_WAIT_SWITCH_OFF;
					}	
				}	
				break;
		
			/* Wait for controller off */
			case STEP_WAIT_SWITCH_OFF:
				if (pAxisData->controller.status == ncOFF)
				{		
					step[i] = STEP_COMMAND;	
				}
				break;
			/* Select movement type */
			case STEP_ERROR_STOP:
				gMultiAxis[i].cmd.negJog = 0;
				gMultiAxis[i].cmd.posJog = 0;
				gMultiAxis[i].cmd.errorAcknowledge = 0;
				/* Stop active movement*/
				if(pAxisData->move.mode != ncOFF)
				{	
					pAxisData->move.stop.index.command = 0;	
					ncStatus = ncaction(axisObj[i],ncMOVE,ncSTOP);
					if(ncStatus == ncOK)
					{
						step[i] = STEP_WAIT_ERROR_STOP;
					}	
				}
				else
				{
					step[i] = STEP_ERROR_SWITCH_OFF;   
				}	           
				break;
			case STEP_WAIT_ERROR_STOP:
				if (pAxisData->move.mode == ncOFF)
				{
					step[i] = STEP_ERROR_SWITCH_OFF;
				}
				break;
			/* Switch controller off if required */
			case STEP_ERROR_SWITCH_OFF:
				ncStatus = ncaction(axisObj[i],ncCONTROLLER,ncSWITCH_OFF);
				if(ncStatus == ncOK)
				{
					step[i] = STEP_WAIT_ERROR_SWITCH_OFF;
				}
				break;
			/* Wait for controller off */
			case STEP_WAIT_ERROR_SWITCH_OFF:
				if(pAxisData->controller.status == ncOFF)
				{
					step[i] = STEP_MESSAGE_TEXT;
				}
				break;
			/* Determine text for current message record */
			case STEP_MESSAGE_TEXT:
				ncStatus = ncaction(axisObj[i], ncMESSAGE, ncTEXT);
				if (ncStatus == ncOK)
				{
					step[i] = STEP_WAIT_MESSAGE_TEXT;
				}
				break;
		
			/* Wait for end of text determination */
			case STEP_WAIT_MESSAGE_TEXT:
				if ((pAxisData->message.text.status.lines != 0) || 
					(pAxisData->message.text.status.error != 0))
				{
					step[i] = STEP_MESSAGE_ACKN;
				}
				break;
			/* Acknowledge current message record */
			case STEP_MESSAGE_ACKN:
				if(gMultiAxis[i].cmd.errorAcknowledge == 1)
				{
					gMultiAxis[i].cmd.errorAcknowledge = 0;
					ncStatus = ncaction(axisObj[i], ncMESSAGE, ncACKNOWLEDGE);
					if (ncStatus == ncOK)
					{	
						if (pAxisData->message.record.number == 0)
						{
							/* all message records acknowledged */
							memset(gMultiAxis[i].alarm.errorText,0,sizeof(gMultiAxis[i].alarm.errorText));
							gMultiAxis[i].status.errorStep = 0;
							if(pAxisData->global.init == ncTRUE)    
							{
								step[i] = STEP_COMMAND;
							}
							else
							{
								step[i] = STEP_WAIT_NET_INIT;
							}
						}
						else
						{
							step[i] = STEP_MESSAGE_TEXT;
						}
					}                
				}
				break;
		}
	}
}
